@@ -16,11 +16,11 @@ from composer.algorithms.blurpool.blurpool_layers import blur_2d as blur_2d
 from composer.algorithms.blurpool.blurpool_layers import blurmax_pool2d as blurmax_pool2d
 
 __all__ = [
-    "BlurPool",
-    "apply_blurpool",
-    "BlurConv2d",
-    "BlurMaxPool2d",
-    "BlurPool2d",
-    "blur_2d",
-    "blurmax_pool2d",
+    'BlurPool',
+    'apply_blurpool',
+    'BlurConv2d',
+    'BlurMaxPool2d',
+    'BlurPool2d',
+    'blur_2d',
+    'blurmax_pool2d',
 ]
