@@ -5,12 +5,14 @@
 
 from __future__ import annotations
 
+import contextlib
 import itertools
 import logging
 from typing import Any, Dict, Optional, Union
 
 import torch
 
+import composer.utils.misc as misc
 from composer.callbacks.checkpoint_saver import CheckpointSaver
 from composer.core import Algorithm, Event, State, Time, TimeUnit
 from composer.loggers import Logger
@@ -56,25 +58,37 @@ def compute_ema(model: torch.nn.Module,
                 ema_model = models.resnet50()
                 cf.compute_ema(model, ema_model, smoothing=0.9)
     """
-    with torch.no_grad():
-        # If the ema model is a pytorch module, can just use the state_dict
-        if isinstance(ema_model, torch.nn.Module):
-            ema_params = ema_model.state_dict()
-            for name, param in itertools.chain(model.named_parameters(), model.named_buffers()):
-                if name in ema_params:
-                    ema_params[name].copy_(ema_params[name] * smoothing + param.data * (1. - smoothing))
-        # Otherwise, the ema model needs to define the named_parameters and named_buffers dictionaries
-        # These should contain the parameters and buffers to average.
-        elif isinstance(ema_model, EMAParameters):
-            ema_parameters = ema_model.named_parameters_dict
-            ema_buffers = ema_model.named_buffers_dict
-            for name, param in itertools.chain(model.named_parameters(), model.named_buffers()):
-                if name in ema_parameters:
-                    ema_parameters[name].copy_(ema_parameters[name] * smoothing + param.data * (1. - smoothing))
-                if name in ema_buffers:
-                    ema_buffers[name].copy_(ema_buffers[name] * smoothing + param.data * (1. - smoothing))
-        else:
-            raise ValueError('ema_model must be a torch.nn.Module or EMAParameters')
+    model_context_manager = get_model_context_manager(model)
+
+    with model_context_manager:
+        with torch.no_grad():
+            # If the ema model is a pytorch module, can just use the state_dict
+            if isinstance(ema_model, torch.nn.Module):
+                ema_params = ema_model.state_dict()
+                for name, param in itertools.chain(model.named_parameters(), model.named_buffers()):
+                    if name in ema_params:
+                        ema_params[name].copy_(ema_params[name] * smoothing + param.data * (1. - smoothing))
+            # Otherwise, the ema model needs to define the named_parameters and named_buffers dictionaries
+            # These should contain the parameters and buffers to average.
+            elif isinstance(ema_model, EMAParameters):
+                ema_parameters = ema_model.named_parameters_dict
+                ema_buffers = ema_model.named_buffers_dict
+                for name, param in itertools.chain(model.named_parameters(), model.named_buffers()):
+                    if name in ema_parameters:
+                        ema_parameters[name].copy_(ema_parameters[name] * smoothing + param.data * (1. - smoothing))
+                    if name in ema_buffers:
+                        ema_buffers[name].copy_(ema_buffers[name] * smoothing + param.data * (1. - smoothing))
+            else:
+                raise ValueError('ema_model must be a torch.nn.Module or EMAParameters')
+
+
+def get_model_context_manager(model: torch.nn.Module):
+    """Summons full params for FSDP, which is required to update sharded params."""
+    fsdp_enabled = misc.is_model_fsdp(model)
+    model_context_manager = contextlib.nullcontext()
+    if fsdp_enabled:
+        model_context_manager = model.module.summon_full_params(model.module)  # type: ignore
+    return model_context_manager
 
 
 class EMA(Algorithm):
@@ -117,6 +131,7 @@ class EMA(Algorithm):
             specified. Default: ``None``.
 
     Example:
+
         .. testcode::
 
             from composer.algorithms import EMA
@@ -183,7 +198,8 @@ class EMA(Algorithm):
             self.smoothing = smoothing
 
         # Construct the appropriate matching events
-        self.match_events = [Event.FIT_START, Event.BATCH_START, Event.EVAL_START, Event.EVAL_END]
+        self.move_device_events = [Event.EVAL_START, Event.FIT_START, Event.PREDICT_START]
+        self.move_param_events = [Event.BATCH_START, Event.EVAL_START, Event.EVAL_END]
         self.checkpoint_events = [Event.BATCH_CHECKPOINT, Event.EPOCH_CHECKPOINT]
         if self.update_interval.unit == TimeUnit.BATCH:
             self.update_event = Event.BATCH_END
@@ -227,11 +243,16 @@ class EMA(Algorithm):
         if event in [Event.BATCH_CHECKPOINT, Event.EPOCH_CHECKPOINT] and self.ema_started:
             checkpoint_savers = [cb for cb in state.callbacks if isinstance(cb, CheckpointSaver)]
             for checkpoint_saver in checkpoint_savers:
+                assert callable(checkpoint_saver.save_interval)
                 if checkpoint_saver.save_interval(state, event) is True:
                     return True
 
-        # Otherwise, always run on some events after ema has started
-        if event in self.match_events and self.ema_started:
+        # Otherwise, always run on events where ema params must be moved after ema has started
+        if event in self.move_param_events and self.ema_started:
+            return True
+
+        # Run on events where ema params must be moved to the correct device
+        if event in self.move_device_events and self.ema_started:
             return True
 
         # Conditionally run on the update event if ema has started
@@ -250,7 +271,7 @@ class EMA(Algorithm):
 
         assert self.ema_model is not None
 
-        if event == Event.FIT_START:
+        if event == Event.FIT_START or event == Event.PREDICT_START:
             # Ensure that params are on the right device if a checkpoint has been loaded
             self.ema_model.move_params_to_device(destination_model=state.model)
 
@@ -262,7 +283,10 @@ class EMA(Algorithm):
             # Update the ema model
             compute_ema(state.model, self.ema_model, smoothing=self.smoothing)
 
-        if event == Event.EVAL_START and self.ema_weights_active is False:
+        if event == Event.EVAL_START:
+            # Verify that the ema params are on the correct device.
+            # Needed to ensure doing eval before training can resume correctly.
+            self.ema_model.move_params_to_device(destination_model=state.model)
             # Swap out the training model for the ema model in state
             self._ensure_ema_weights_active(state)
 
@@ -370,11 +394,13 @@ class EMAParameters:
 
     def __init__(self, model: Union[None, torch.nn.Module]):
         if model is not None:
-            # Copy the trainable parameters and buffers.
-            self.named_parameters_dict = {
-                name: param.data.clone() for name, param in model.named_parameters() if param.requires_grad
-            }
-            self.named_buffers_dict = {name: buffer.data.clone() for name, buffer in model.named_buffers()}
+            model_context_manager = get_model_context_manager(model)
+            with model_context_manager:
+                # Copy the trainable parameters and buffers.
+                self.named_parameters_dict = {
+                    name: param.data.clone() for name, param in model.named_parameters() if param.requires_grad
+                }
+                self.named_buffers_dict = {name: buffer.data.clone() for name, buffer in model.named_buffers()}
         else:
             # Empty storage
             self.named_parameters_dict = {}
@@ -388,32 +414,50 @@ class EMAParameters:
 
     def swap_params(self, model: torch.nn.Module):
         """Swaps the parameters and buffers of a model with the ema parameters."""
+        model_context_manager = get_model_context_manager(model)
+
         with torch.no_grad():
             ema_params = self.named_parameters_dict
             ema_buffers = self.named_buffers_dict
 
-            for name, param in model.named_parameters():
-                if name in ema_params:
-                    param.data, ema_params[name] = ema_params[name], param.data
+            with model_context_manager:
+                for name, param in model.named_parameters():
+                    if name in ema_params:
+                        # Use copy instead of raw data access (eg .data) doesn't work with FSDP
+                        dummy_param = param.clone()
+                        param.copy_(ema_params[name])
+                        ema_params[name].copy_(dummy_param)
 
-            for name, buffer in model.named_buffers():
-                buffer.data, ema_buffers[name] = ema_buffers[name], buffer.data
+                for name, buffer in model.named_buffers():
+                    if name in ema_buffers:
+                        # Use copy instead of raw data access (eg .data) doesn't work with FSDP
+                        dummy_buffer = buffer.clone()
+                        buffer.copy_(ema_buffers[name])
+                        ema_buffers[name].copy_(dummy_buffer)
 
     def transfer_ema_params(self, model: torch.nn.Module):
         """Transfers the parameters and buffers from the ema model to the supplied model."""
-        with torch.no_grad():
-            for name, param in model.named_parameters():
-                if name in self.named_parameters_dict:
-                    param.data = self.named_parameters_dict[name]
+        model_context_manager = get_model_context_manager(model)
 
-            for name, buffer in model.named_buffers():
-                buffer.data = self.named_buffers_dict[name]
+        with model_context_manager:
+            with torch.no_grad():
+                for name, param in model.named_parameters():
+                    if name in self.named_parameters_dict:
+                        param.copy_(self.named_parameters_dict[name])
+
+                for name, buffer in model.named_buffers():
+                    if name in self.named_buffers_dict:
+                        buffer.copy_(self.named_buffers_dict[name])
 
     def move_params_to_device(self, destination_model: torch.nn.Module):
         """Moves the ema parameters and buffers to the device of a destination model."""
-        model_state_dict = destination_model.state_dict()
-        for name, param in self.named_parameters_dict.items():
-            self.named_parameters_dict[name] = param.to(model_state_dict[name].device)
+        model_context_manager = get_model_context_manager(destination_model)
 
-        for name, buffer in self.named_buffers_dict.items():
-            self.named_buffers_dict[name] = buffer.to(model_state_dict[name].device)
+        with model_context_manager:
+            for name, param in destination_model.named_parameters():
+                if name in self.named_parameters_dict:
+                    self.named_parameters_dict[name] = self.named_parameters_dict[name].to(param.device)
+
+            for name, buffer in destination_model.named_buffers():
+                if name in self.named_buffers_dict:
+                    self.named_buffers_dict[name] = self.named_buffers_dict[name].to(buffer.device)
