@@ -9,4 +9,4 @@ See the :doc:`Method Card </method_cards/layer_freezing>` for more details.
 from composer.algorithms.layer_freezing.layer_freezing import LayerFreezing as LayerFreezing
 from composer.algorithms.layer_freezing.layer_freezing import freeze_layers as freeze_layers
 
-__all__ = ["LayerFreezing", "freeze_layers"]
+__all__ = ['LayerFreezing', 'freeze_layers']
