@@ -6,6 +6,7 @@
 from __future__ import annotations
 
 import logging
+import warnings
 from typing import Any, Dict, List, Optional
 
 import torch
@@ -115,7 +116,7 @@ class SWA(Algorithm):
                  anneal_steps: int = 10,
                  swa_lr: Optional[float] = None):
 
-        log.warning(
+        warnings.warn(
             'SWA has known issues when resuming from a checkpoint on multiple GPUs, which will cause an error when resuming without `load_weights_only=True`.'
         )
         self.schedule_swa_lr = schedule_swa_lr
