@@ -8,70 +8,26 @@ from __future__ import annotations
 import logging
 import os
 import pathlib
+import shutil
 import tempfile
 import textwrap
+from pathlib import Path
 from typing import Callable, List, Optional, Union
 
-from composer.core import Callback, Event, State, Time, TimeUnit
+from composer.callbacks.utils import create_interval_scheduler
+from composer.core import Callback, Event, State, Time
 from composer.loggers import Logger
 from composer.utils import (FORMAT_NAME_WITH_DIST_AND_TIME_TABLE, FORMAT_NAME_WITH_DIST_TABLE, PartialFilePath,
                             checkpoint, create_symlink_file, dist, ensure_folder_has_no_conflicting_files,
-                            format_name_with_dist, is_model_deepspeed, reproducibility)
+                            format_name_with_dist, format_name_with_dist_and_time, is_model_deepspeed, reproducibility)
+from composer.utils.checkpoint import _TORCH_DISTRIBUTED_CHECKPOINTS_FILENAME
+from composer.utils.misc import using_torch_2
 
 log = logging.getLogger(__name__)
 
-__all__ = ['CheckpointSaver', 'checkpoint_periodically']
+__all__ = ['CheckpointSaver']
 
-
-def checkpoint_periodically(interval: Union[str, int, Time]) -> Callable[[State, Event], bool]:
-    r"""Helper function to create a checkpoint scheduler according to a specified interval.
-
-    Args:
-        interval (Union[str, int, :class:`.Time`]): The interval describing how often checkpoints should be
-            saved. If an integer, it will be assumed to be in :attr:`.TimeUnit.EPOCH`\s.
-            Otherwise, the unit must be either :attr:`.TimeUnit.EPOCH` or :attr:`.TimeUnit.BATCH`.
-
-            Checkpoints will be saved every ``n`` batches or epochs (depending on the unit),
-            and at the end of training.
-
-    Returns:
-        Callable[[State, Event], bool]: A function that can be passed as the ``save_interval``
-            argument into the :class:`.CheckpointSaver`.
-    """
-    if isinstance(interval, str):
-        interval = Time.from_timestring(interval)
-    if isinstance(interval, int):
-        interval = Time(interval, TimeUnit.EPOCH)
-
-    if interval.unit == TimeUnit.EPOCH:
-        save_event = Event.EPOCH_CHECKPOINT
-    elif interval.unit == TimeUnit.BATCH:
-        save_event = Event.BATCH_CHECKPOINT
-    else:
-        raise NotImplementedError(
-            f'Unknown checkpointing interval: {interval.unit}. Must be TimeUnit.EPOCH or TimeUnit.BATCH.')
-
-    def save_interval(state: State, event: Event):
-        elapsed_duration = state.get_elapsed_duration()
-        assert elapsed_duration is not None, 'elapsed_duration is set on the BATCH_CHECKPOINT and EPOCH_CHECKPOINT'
-
-        # Always checkpoint at end of training
-        if elapsed_duration >= 1.0:
-            return True
-
-        if save_event == Event.EPOCH_CHECKPOINT:
-            count = state.timestamp.epoch
-        elif save_event == Event.BATCH_CHECKPOINT:
-            count = state.timestamp.batch
-        else:
-            raise RuntimeError(f'Invalid save_event: {save_event}')
-
-        if event == save_event and int(count) % int(interval) == 0:
-            return True
-
-        return False
-
-    return save_interval
+_TORCH_DISTRIBUTED_CHECKPOINTS_METADATA_FILENAME = '.metadata'
 
 
 class CheckpointSaver(Callback):  # noqa: D101
@@ -296,7 +252,7 @@ class CheckpointSaver(Callback):  # noqa: D101
         latest_remote_file_name = str(latest_remote_file_name) if latest_remote_file_name is not None else None
 
         if not callable(save_interval):
-            save_interval = checkpoint_periodically(save_interval)
+            save_interval = create_interval_scheduler(save_interval)
         self.save_interval = save_interval
         self.last_checkpoint_batch: Optional[Time] = None
 
@@ -304,7 +260,6 @@ class CheckpointSaver(Callback):  # noqa: D101
 
         self.filename = PartialFilePath(filename.lstrip('/'), folder)
         self.latest_filename = PartialFilePath(latest_filename.lstrip('/'), folder) if latest_filename else None
-
         self.remote_file_name = PartialFilePath(remote_file_name) if remote_file_name else None
         self.latest_remote_file_name = PartialFilePath(latest_remote_file_name) if latest_remote_file_name else None
 
@@ -313,6 +268,8 @@ class CheckpointSaver(Callback):  # noqa: D101
         self.num_checkpoints_to_keep = num_checkpoints_to_keep
         self.weights_only = weights_only
 
+        self.start_batch = None
+
     def init(self, state: State, logger: Logger) -> None:
         folder = format_name_with_dist(self.folder, state.run_name)
         os.makedirs(folder, exist_ok=True)
@@ -329,7 +286,10 @@ class CheckpointSaver(Callback):  # noqa: D101
         if is_model_deepspeed(state.model) and self.weights_only:
             raise NotImplementedError('weights_only=True is not supported when using DeepSpeed.')
 
+        self.start_batch = state.timestamp.batch
+
     def batch_checkpoint(self, state: State, logger: Logger):
+        assert callable(self.save_interval)
         if self.save_interval(state, Event.BATCH_CHECKPOINT) and self.last_checkpoint_batch != state.timestamp.batch:
             self._save_checkpoint(
                 state,
@@ -337,12 +297,21 @@ class CheckpointSaver(Callback):  # noqa: D101
             )
 
     def epoch_checkpoint(self, state: State, logger: Logger):
+        assert callable(self.save_interval)
         if self.save_interval(state, Event.EPOCH_CHECKPOINT) and self.last_checkpoint_batch != state.timestamp.batch:
             self._save_checkpoint(
                 state,
                 logger,
             )
 
+    def close(self, state: State, logger: Logger):
+        trained_at_least_one_batch = self.start_batch is not None and self.start_batch != state.timestamp.batch
+        if self.last_checkpoint_batch != state.timestamp.batch and trained_at_least_one_batch:
+            self._save_checkpoint(
+                state,
+                logger,
+            )
+
     def get_state_dict(self, state):
         return {
             'state': state.state_dict(),
@@ -358,35 +327,73 @@ class CheckpointSaver(Callback):  # noqa: D101
             raise ValueError(f'Save filename {self.filename.filename} must have {{rank}} for deepspeed.')
 
         # save the checkpoint to the filename
-        filename = self.filename.format(state, is_deepspeed)
+        filename_with_placeholders = self.filename.format(state, is_deepspeed, keep_placeholders=True)
 
         saved_path = checkpoint.save_checkpoint(
             state=state,
-            filename=filename,
+            filename=filename_with_placeholders,
             weights_only=self.weights_only,
         )
+        log.debug(f'Checkpoint locally saved to {saved_path}')
 
         if not saved_path:  # not all ranks save
             return
+        metadata_local_file_path = None
+        if dist.get_global_rank() == 0 and state.fsdp_elastic_sharded_enabled:
+            metadata_local_file_path = format_name_with_dist_and_time(
+                os.path.join(Path(saved_path).parent, _TORCH_DISTRIBUTED_CHECKPOINTS_METADATA_FILENAME), state.run_name,
+                state.timestamp)
 
-        if self.latest_filename is not None:
+        if self.latest_filename is not None and self.num_checkpoints_to_keep != 0:
             symlink = self.latest_filename.format(state, is_deepspeed)
             os.makedirs(os.path.dirname(symlink), exist_ok=True)
             try:
                 os.remove(symlink)
             except FileNotFoundError:
                 pass
-            os.symlink(os.path.relpath(filename, os.path.dirname(symlink)), symlink)
+            # Sharded checkpoints for torch >2.0 use directories not files for load_paths
+            if state.fsdp_elastic_sharded_enabled:
+                src_path = str(pathlib.Path(saved_path).parent)
+            else:
+                src_path = saved_path
+            this_rank_saves_symlinks = dist.get_global_rank() == 0 or not state.fsdp_elastic_sharded_enabled
+            if this_rank_saves_symlinks:
+                os.symlink(os.path.relpath(src_path, os.path.dirname(symlink)), symlink)
 
         # if remote file name provided, upload the checkpoint
         if self.remote_file_name is not None:
-            remote_file_name = self.remote_file_name.format(
-                state,
-                is_deepspeed,
-            ).lstrip('/')
+            if state.fsdp_sharded_state_dict_enabled:
+                remote_file_name = self.remote_file_name.format(
+                    state,
+                    is_deepspeed,
+                    keep_placeholders=True,
+                ).lstrip('/')
+                assert state.sharded_ckpt_prefix_dir is not None
+                remote_prefix = state.sharded_ckpt_prefix_dir
+                ckpt_filename = _TORCH_DISTRIBUTED_CHECKPOINTS_FILENAME if using_torch_2() else pathlib.Path(
+                    remote_file_name).name
+                remote_file_name = os.path.join(pathlib.Path(remote_file_name).parent, remote_prefix, ckpt_filename)
+                remote_file_name = format_name_with_dist_and_time(remote_file_name, state.run_name, state.timestamp)
+                # Upload metadata file.
+                # The metadata file contains info related to which shards are saved where.
+                if dist.get_global_rank() == 0 and state.fsdp_elastic_sharded_enabled:
+                    metadata_remote_file_name = format_name_with_dist_and_time(
+                        os.path.join(Path(remote_file_name).parent, _TORCH_DISTRIBUTED_CHECKPOINTS_METADATA_FILENAME),
+                        state.run_name, state.timestamp)
+                    assert metadata_local_file_path is not None
+                    logger.upload_file(remote_file_name=metadata_remote_file_name,
+                                       file_path=metadata_local_file_path,
+                                       overwrite=self.overwrite)
+            else:
+                remote_file_name = self.remote_file_name.format(
+                    state,
+                    is_deepspeed,
+                ).lstrip('/')
 
-            logger.upload_file(remote_file_name=remote_file_name, file_path=filename, overwrite=self.overwrite)
+            log.debug(f'Uploading checkpoint to {remote_file_name}')
+            logger.upload_file(remote_file_name=remote_file_name, file_path=saved_path, overwrite=self.overwrite)
 
+            # symlinks stay the same with sharded checkpointing
             if self.latest_remote_file_name is not None:
                 symlink_name = self.latest_remote_file_name.format(
                     state,
@@ -396,19 +403,34 @@ class CheckpointSaver(Callback):  # noqa: D101
                 # create and upload a symlink file
                 with tempfile.TemporaryDirectory() as tmpdir:
                     symlink_filename = os.path.join(tmpdir, 'latest.symlink')
-                    create_symlink_file(remote_file_name, symlink_filename)
-                    logger.upload_file(
-                        remote_file_name=symlink_name,
-                        file_path=symlink_filename,
-                        overwrite=True,
-                    )
+                    # Sharded checkpoints for torch >2.0 use directories not files for load_paths
+                    if state.fsdp_elastic_sharded_enabled:
+                        src_path = str(pathlib.Path(remote_file_name).parent)
+                    else:
+                        src_path = remote_file_name
+                    log.debug(f'Creating symlink file {symlink_filename} -> {src_path}')
+                    this_rank_saves_symlinks = dist.get_global_rank() == 0 or not state.fsdp_elastic_sharded_enabled
+                    if this_rank_saves_symlinks:
+                        create_symlink_file(src_path, symlink_filename)
+                        logger.upload_file(
+                            remote_file_name=symlink_name,
+                            file_path=symlink_filename,
+                            overwrite=True,
+                        )
 
-        self.saved_checkpoints.append(filename)
+        self.saved_checkpoints.append(saved_path)
 
         if self.num_checkpoints_to_keep >= 0:
-            self._rotate_checkpoints()
+            self._rotate_checkpoints(sharding_enabled=state.fsdp_sharded_state_dict_enabled)
+
+    def _rotate_checkpoints(self, sharding_enabled: bool = False):
 
-    def _rotate_checkpoints(self):
         while len(self.saved_checkpoints) > self.num_checkpoints_to_keep:
+            prefix_dir = None
             checkpoint = self.saved_checkpoints.pop(0)
-            os.remove(checkpoint)
+            prefix_dir = str(Path(checkpoint).parent)
+            if not sharding_enabled:
+                os.remove(checkpoint)
+            else:
+                if dist.get_global_rank() == 0:
+                    shutil.rmtree(prefix_dir)
