@@ -51,19 +51,27 @@ class MemoryMonitor(Callback):
 
     The following statistics are recorded:
 
-    +----------------+-----------------------------------------------------------------------------------+
-    | Statistic      | Description                                                                       |
-    +================+===================================================================================+
-    | allocated_mem  | Amount of allocated memory in gigabytes.                                          |
-    +----------------+-----------------------------------------------------------------------------------+
-    | active_mem     | Amount of active memory in gigabytes at the time of recording.                    |
-    +----------------+-----------------------------------------------------------------------------------+
-    | inactive_mem   | Amount of inactive, non-releaseable memory in gigabytes at the time of recording. |
-    +----------------+-----------------------------------------------------------------------------------+
-    | reserved_mem   | Amount of reserved memory in gigabytes at the time of recording.                  |
-    +----------------+-----------------------------------------------------------------------------------+
-    | alloc_retries  | Number of failed cudaMalloc calls that result in a cache flush and retry.         |
-    +----------------+-----------------------------------------------------------------------------------+
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | Statistic              | Description                                                                               |
+    +========================+===========================================================================================+
+    | current_allocated_mem  | Current amount of allocated memory in gigabytes.                                          |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | current_active_mem     | Current amount of active memory in gigabytes at the time of recording.                    |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | current_inactive_mem   | Current amount of inactive, non-releaseable memory in gigabytes at the time of recording. |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | current_reserved_mem   | Current amount of reserved memory in gigabytes at the time of recording.                  |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | peak_allocated_mem     | Peak amount of allocated memory in gigabytes.                                             |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | peak_active_mem        | Peak amount of active memory in gigabytes at the time of recording.                       |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | peak_inactive_mem      | Peak amount of inactive, non-releaseable memory in gigabytes at the time of recording.    |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | peak_reserved_mem      | Peak amount of reserved memory in gigabytes at the time of recording.                     |
+    +------------------------+-------------------------------------------------------------------------------------------+
+    | alloc_retries          | Number of failed cudaMalloc calls that result in a cache flush and retry.                 |
+    +------------------------+-------------------------------------------------------------------------------------------+
 
     .. note::
         Memory usage monitoring is only supported for GPU devices.
@@ -98,10 +106,14 @@ class MemoryMonitor(Callback):
 
 
 _MEMORY_KEYS = {
-    'allocated_bytes.all.current': 'allocated_mem',
-    'active_bytes.all.current': 'active_mem',
-    'inactive_split_bytes.all.current': 'inactive_mem',
-    'reserved_bytes.all.current': 'reserved_mem',
+    'allocated_bytes.all.current': 'current_allocated_mem',
+    'active_bytes.all.current': 'current_active_mem',
+    'inactive_split_bytes.all.current': 'current_inactive_mem',
+    'reserved_bytes.all.current': 'current_reserved_mem',
+    'allocated_bytes.all.peak': 'peak_allocated_mem',
+    'active_bytes.all.peak': 'peak_active_mem',
+    'inactive_split_bytes.all.peak': 'peak_inactive_mem',
+    'reserved_bytes.all.peak': 'peak_reserved_mem',
     'num_alloc_retries': 'alloc_retries',
 }
 
