@@ -53,40 +53,24 @@ class OptimizerMonitor(Callback):
     | ``l2_norm/moment/LAYER_NAME``                 |  calling optimizer step.                            |
     |                                               |                                                     |
     +-----------------------------------------------+-----------------------------------------------------+
-    |                                               | Layer-wise ratio of the gradient norm to the        |
-    | ``l2_norm_ratio/moment_grad/LAYER_NAME``      | moment norm after calling optimizer step.           |
-    |                                               |                                                     |
-    +-----------------------------------------------+-----------------------------------------------------+
-    |                                               | Layer-wise cosine angle between gradient and moment |
-    | ``cosine/moment_grad/LAYER_NAME``             | after calling optimizer step.                       |
-    |                                               |                                                     |
-    +-----------------------------------------------+-----------------------------------------------------+
     |                                               | Layer-wise L2 norms of parameter weights            |
     | ``l2_norm/param/LAYER_NAME``                  |                                                     |
     |                                               |                                                     |
     +-----------------------------------------------+-----------------------------------------------------+
-    |                                               | Layer-wise L2 norms of the square root              |
-    | ``l2_norm/second_moment_sqrt/LAYER_NAME``     |  of the Adam second moment is.                      |
-    |                                               |                                                     |
-    +-----------------------------------------------+-----------------------------------------------------+
     |                                               | Layer-wise L2 norms of the step                     |
     | ``l2_norm/update/LAYER_NAME``                 |                                                     |
     |                                               |                                                     |
     +-----------------------------------------------+-----------------------------------------------------+
-    |                                               | Layer-wise cosine between the gradient and the step |
-    | ``cosine/update_grad/LAYER_NAME``             |                                                     |
-    |                                               |                                                     |
-    +-----------------------------------------------+-----------------------------------------------------+
-    |                                               | Layer-wise ratio between step size and parameter    |
-    | ``l2_norm_ratio/update_param/LAYER_NAME``     | norm                                                |
-    |                                               |                                                     |
-    +-----------------------------------------------+-----------------------------------------------------+
     """
 
-    def __init__(self, log_optimizer_metrics: bool = True):
+    def __init__(self, log_optimizer_metrics: bool = True, batch_log_interval: int = 10):
         self.log_optimizer_metrics = log_optimizer_metrics
+        self.batch_log_interval = batch_log_interval
 
     def batch_end(self, state: State, logger: Logger):
+        if state.timestamp.batch.value % self.batch_log_interval != 0:
+            return
+
         norm = 0.0
         optimizer_metrics = {}
 
@@ -95,7 +79,7 @@ class OptimizerMonitor(Callback):
 
                 metric_reporter = getattr(state.optimizers[0], 'report_per_parameter_metrics', None)
                 if callable(metric_reporter) and self.log_optimizer_metrics:
-                    optimizer_metrics = metric_reporter(p, name, optimizer_metrics)
+                    optimizer_metrics.update(metric_reporter(p, name, optimizer_metrics))
 
                 # Always log grad norm as a default metric if it's not specified
                 if f'l2_norm/grad/{name}' not in optimizer_metrics:
