@@ -6,7 +6,8 @@
 from composer.devices.device import Device
 from composer.devices.device_cpu import DeviceCPU
 from composer.devices.device_gpu import DeviceGPU
+from composer.devices.device_hpu import DeviceHPU
 from composer.devices.device_mps import DeviceMPS
 from composer.devices.device_tpu import DeviceTPU
 
-__all__ = ['Device', 'DeviceCPU', 'DeviceGPU', 'DeviceMPS', 'DeviceTPU']
+__all__ = ['Device', 'DeviceCPU', 'DeviceGPU', 'DeviceMPS', 'DeviceTPU', 'DeviceHPU']
