@@ -20,13 +20,16 @@ from multiprocessing.context import SpawnProcess
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 from urllib.parse import urlparse
 
+import torch
+
 from composer.loggers.logger import Logger
 from composer.loggers.logger_destination import LoggerDestination
-from composer.utils import (LibcloudObjectStore, ObjectStore, ObjectStoreTransientError, OCIObjectStore, S3ObjectStore,
-                            SFTPObjectStore, dist, format_name_with_dist, get_file, retry)
+from composer.utils import (GCSObjectStore, LibcloudObjectStore, ObjectStore, ObjectStoreTransientError, OCIObjectStore,
+                            S3ObjectStore, SFTPObjectStore, dist, format_name_with_dist, get_file, retry)
 
 if TYPE_CHECKING:
     from composer.core import State
+    from composer.devices import Device
 
 log = logging.getLogger(__name__)
 
@@ -38,7 +41,8 @@ def _build_remote_backend(remote_backend_name: str, backend_kwargs: Dict[str, An
         's3': S3ObjectStore,
         'oci': OCIObjectStore,
         'sftp': SFTPObjectStore,
-        'libcloud': LibcloudObjectStore
+        'libcloud': LibcloudObjectStore,
+        'gs': GCSObjectStore,
     }
     remote_backend_cls = remote_backend_name_to_cls.get(remote_backend_name, None)
     if remote_backend_cls is None:
@@ -225,7 +229,7 @@ class RemoteUploaderDownloader(LoggerDestination):
         parsed_remote_bucket = urlparse(bucket_uri)
         self.remote_backend_name, self.remote_bucket_name = parsed_remote_bucket.scheme, parsed_remote_bucket.netloc
         self.backend_kwargs = backend_kwargs if backend_kwargs is not None else {}
-        if self.remote_backend_name in ['s3', 'oci'] and 'bucket' not in self.backend_kwargs:
+        if self.remote_backend_name in ['s3', 'oci', 'gs'] and 'bucket' not in self.backend_kwargs:
             self.backend_kwargs['bucket'] = self.remote_bucket_name
         elif self.remote_backend_name == 'sftp' and 'host' not in self.backend_kwargs:
             self.backend_kwargs['host'] = f'sftp://{self.remote_bucket_name}'
@@ -441,15 +445,15 @@ class RemoteUploaderDownloader(LoggerDestination):
                  progress_bar=progress_bar)
 
     def fit_end(self, state: State, logger: Logger):
-        self.wait_for_workers()
+        self.wait_for_workers(state.device)
 
-    def eval_end(self, state: State, logger: Logger):
-        self.wait_for_workers()
+    def eval_standalone_end(self, state: State, logger: Logger):
+        self.wait_for_workers(state.device)
 
     def predict_end(self, state: State, logger: Logger):
-        self.wait_for_workers()
+        self.wait_for_workers(state.device)
 
-    def wait_for_workers(self):
+    def wait_for_workers(self, device: Device):
         """Wait for all tasks to be completed.
 
         This is called after fit/eval/predict. If we don't wait, then a worker might not schedule
@@ -463,9 +467,21 @@ class RemoteUploaderDownloader(LoggerDestination):
                 if len(self._logged_objects) == 0:
                     break
             time.sleep(0.2)  # Yield lock for enqueue thread
+
         # Verify all tasks have been completed unless a worker threw an exception
-        while not self._file_upload_queue.empty() and self._exception_queue.empty():
-            time.sleep(0.2)
+        all_ranks_upload_done_tensor = device.tensor_to_device(
+            torch.tensor([int(not self._file_upload_queue.empty() and self._exception_queue.empty())],
+                         dtype=torch.uint8))
+        dist.all_reduce(all_ranks_upload_done_tensor, reduce_operation='MAX')
+        upload_not_done = all_ranks_upload_done_tensor.item() == 1
+        while upload_not_done:
+            time.sleep(2)
+            all_ranks_upload_done_tensor = device.tensor_to_device(
+                torch.tensor([int(not self._file_upload_queue.empty() and self._exception_queue.empty())],
+                             dtype=torch.uint8))
+            dist.all_reduce(all_ranks_upload_done_tensor, reduce_operation='MAX')
+            upload_not_done = all_ranks_upload_done_tensor.item() == 1
+
         if not self._exception_queue.empty():
             e = self._exception_queue.get_nowait()
             raise e
