@@ -30,6 +30,9 @@ class MIoU(Metric):
         ignore_index (int, optional): the index to ignore when computing mIoU. Default: ``-1``.
     """
 
+    # Make torchmetrics call update only once
+    full_state_update = False
+
     def __init__(self, num_classes: int, ignore_index: int = -1):
         super().__init__(dist_sync_on_step=True)
         self.num_classes = num_classes
@@ -71,6 +74,9 @@ class Dice(Metric):
         num_classes (int): the number of classes in the segmentation task.
     """
 
+    # Make torchmetrics call update only once
+    full_state_update = False
+
     def __init__(self, num_classes: int):
         super().__init__(dist_sync_on_step=True)
         self.add_state('n_updates', default=torch.zeros(1), dist_reduce_fx='sum')
@@ -138,6 +144,9 @@ class CrossEntropy(Metric):
         dist_sync_on_step (bool, optional): sync distributed metrics every step. Default: ``False``.
     """
 
+    # Make torchmetrics call update only once
+    full_state_update = False
+
     def __init__(self, ignore_index: int = -100, dist_sync_on_step: bool = False):
         super().__init__(dist_sync_on_step=dist_sync_on_step)
         self.ignore_index = ignore_index
@@ -168,6 +177,9 @@ class LossMetric(Metric):
         dist_sync_on_step (bool, optional): sync distributed metrics every step. Default: ``False``.
     """
 
+    # Make torchmetrics call update only once
+    full_state_update = False
+
     def __init__(self, loss_function: Callable, dist_sync_on_step: bool = False):
         super().__init__(dist_sync_on_step=dist_sync_on_step)
         self.loss_function = loss_function
