@@ -20,7 +20,7 @@ import tqdm
 
 from composer.utils import dist
 from composer.utils.iter_helpers import iterate_with_callback
-from composer.utils.object_store import LibcloudObjectStore, ObjectStore, OCIObjectStore, S3ObjectStore
+from composer.utils.object_store import GCSObjectStore, ObjectStore, OCIObjectStore, S3ObjectStore
 
 if TYPE_CHECKING:
     from composer.core import Timestamp
@@ -346,16 +346,7 @@ def maybe_create_object_store_from_uri(uri: str) -> Optional[ObjectStore]:
         raise NotImplementedError(f'There is no implementation for WandB load_object_store via URI. Please use '
                                   'WandBLogger')
     elif backend == 'gs':
-        if 'GCS_KEY' not in os.environ or 'GCS_SECRET' not in os.environ:
-            raise ValueError(
-                'You must set the GCS_KEY and GCS_SECRET env variable with you HMAC access id and secret respectively')
-
-        return LibcloudObjectStore(
-            provider='google_storage',
-            container=bucket_name,
-            key_environ='GCS_KEY',  # Name of env variable for HMAC access id.
-            secret_environ='GCS_SECRET',  # Name of env variable for HMAC secret.
-        )
+        return GCSObjectStore(bucket=bucket_name)
     elif backend == 'oci':
         return OCIObjectStore(bucket=bucket_name)
     else:
@@ -390,22 +381,9 @@ def maybe_create_remote_uploader_downloader_from_uri(
             warnings.warn(
                 f'There already exists a RemoteUploaderDownloader object to handle the uri: {uri} you specified')
             return None
-    if backend in ['s3', 'oci']:
+    if backend in ['s3', 'oci', 'gs']:
         return RemoteUploaderDownloader(bucket_uri=f'{backend}://{bucket_name}')
 
-    elif backend == 'gs':
-        if 'GCS_KEY' not in os.environ or 'GCS_SECRET' not in os.environ:
-            raise ValueError(
-                'You must set the GCS_KEY and GCS_SECRET env variable with you HMAC access id and secret respectively')
-        return RemoteUploaderDownloader(
-            bucket_uri=f'libcloud://{bucket_name}',
-            backend_kwargs={
-                'provider': 'google_storage',
-                'container': bucket_name,
-                'key_environ': 'GCS_KEY',  # Name of env variable for HMAC access id.
-                'secret_environ': 'GCS_SECRET',  # Name of env variable for HMAC secret.
-            })
-
     elif backend == 'wandb':
         raise NotImplementedError(f'There is no implementation for WandB via URI. Please use '
                                   'WandBLogger with log_artifacts set to True')
@@ -415,13 +393,11 @@ def maybe_create_remote_uploader_downloader_from_uri(
                                   's3 or one of the supported RemoteUploaderDownloader object stores')
 
 
-def get_file(
-    path: str,
-    destination: str,
-    object_store: Optional[Union[ObjectStore, LoggerDestination]] = None,
-    overwrite: bool = False,
-    progress_bar: bool = True,
-):
+def get_file(path: str,
+             destination: str,
+             object_store: Optional[Union[ObjectStore, LoggerDestination]] = None,
+             overwrite: bool = False,
+             progress_bar: bool = True):
     """Get a file from a local folder, URL, or object store.
 
     Args:
@@ -481,13 +457,11 @@ def get_file(
                 log.debug(f'Read path {real_path} from symlink file.')
 
         # Recurse
-        return get_file(
-            path=real_path,
-            destination=destination,
-            object_store=object_store,
-            overwrite=overwrite,
-            progress_bar=progress_bar,
-        )
+        return get_file(path=real_path,
+                        destination=destination,
+                        object_store=object_store,
+                        overwrite=overwrite,
+                        progress_bar=progress_bar)
 
     try:
         _get_file(
@@ -501,13 +475,11 @@ def get_file(
         new_path = path + '.symlink'
         try:
             # Follow the symlink
-            return get_file(
-                path=new_path,
-                destination=destination,
-                object_store=object_store,
-                overwrite=overwrite,
-                progress_bar=progress_bar,
-            )
+            return get_file(path=new_path,
+                            destination=destination,
+                            object_store=object_store,
+                            overwrite=overwrite,
+                            progress_bar=progress_bar)
         except FileNotFoundError as ee:
             # Raise the original not found error first, which contains the path to the user-specified file
             raise e from ee
@@ -578,6 +550,10 @@ def _get_file(
     # It's a local filepath
     if not os.path.exists(path):
         raise FileNotFoundError(f'Local path {path} does not exist')
+
+    if os.path.exists(destination) and overwrite:
+        os.remove(destination)
+
     os.symlink(os.path.abspath(path), destination)
 
 
