@@ -3,6 +3,7 @@
 
 """Object store base class and implementations."""
 
+from composer.utils.object_store.gcs_object_store import GCSObjectStore
 from composer.utils.object_store.libcloud_object_store import LibcloudObjectStore
 from composer.utils.object_store.object_store import ObjectStore, ObjectStoreTransientError
 from composer.utils.object_store.oci_object_store import OCIObjectStore
@@ -11,5 +12,5 @@ from composer.utils.object_store.sftp_object_store import SFTPObjectStore
 
 __all__ = [
     'ObjectStore', 'ObjectStoreTransientError', 'LibcloudObjectStore', 'S3ObjectStore', 'SFTPObjectStore',
-    'OCIObjectStore'
+    'OCIObjectStore', 'GCSObjectStore'
 ]
