@@ -6,7 +6,7 @@ import io
 import os
 import pathlib
 import uuid
-from typing import Any, Callable, Dict, Optional, Union
+from typing import Any, Callable, Dict, List, Optional, Union
 
 from requests.exceptions import ConnectionError
 from urllib3.exceptions import ProtocolError
@@ -170,6 +170,9 @@ class LibcloudObjectStore(ObjectStore):
             # If the file already exits, short-circuit and skip the download
             raise FileExistsError(f'filename {filename} exists and overwrite was set to False.')
 
+        dirname = os.path.dirname(filename)
+        if dirname:
+            os.makedirs(dirname, exist_ok=True)
         obj = self._get_object(object_name)
         # Download first to a tempfile, and then rename, in case if the file gets corrupted in transit
         tmp_filepath = str(filename) + f'.{uuid.uuid4()}.tmp'
@@ -192,6 +195,12 @@ class LibcloudObjectStore(ObjectStore):
         else:
             os.rename(tmp_filepath, filename)
 
+    def list_objects(self, prefix: Optional[str] = None) -> List[str]:
+        if prefix is None:
+            prefix = ''
+
+        return [obj.name for obj in self._provider.list_container_objects(self._container, prefix=prefix)]
+
 
 def _file_to_iterator(f: io.IOBase, chunk_size: int):
     while True:
